/// <reference types="node" />
import * as crypto from 'crypto';
import { TSCharset } from './tscharset';
import { TSData } from './tsdata';
import { Nullable, StringEncoding, TSDataLike, uint32, UUID } from './types';
export declare function $uuid(): UUID;
export declare enum HashMethod {
    SHA256 = "SHA256",
    SHA384 = "SHA384",
    SHA512 = "SHA512"
}
export interface $encryptOptions {
    encoding?: Nullable<StringEncoding | TSCharset>;
}
export declare function $encrypt(ssource: string | TSDataLike, skey: string | TSDataLike | crypto.KeyObject, opts?: Nullable<$encryptOptions>): string | null;
export interface $decryptOptions extends $encryptOptions {
    dataOutput?: Nullable<boolean>;
}
export declare function $crc32(ssource: string | TSDataLike, encoding?: Nullable<StringEncoding | TSCharset>): uint32;
export declare function $decrypt(source: string, skey: string | TSDataLike | crypto.KeyObject, opts?: Nullable<$decryptOptions>): TSData | string | null;
export declare function $hash(buf: string | TSDataLike, method?: Nullable<HashMethod>, encoding?: Nullable<StringEncoding | TSCharset>): string | null;
export declare function $hashfile(filePath: Nullable<string>, method?: HashMethod): Promise<string | null>;
interface PasswordOptions {
    hasLowercase?: boolean;
    hasUppercase?: boolean;
    hasNumeric?: boolean;
    hasSpecials?: boolean;
}
export declare function $random(max: number): number;
export declare function $password(len: number, opts?: PasswordOptions): string | null;
declare global {
    export interface Uint8Array {
        crc32: (this: any) => uint32;
        hash: (this: any, method?: Nullable<HashMethod>) => string | null;
    }
    export interface ArrayBuffer {
        crc32: (this: any) => uint32;
        hash: (this: any, method?: Nullable<HashMethod>) => string | null;
    }
}
export {};
