"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.$dataAspect = exports.$encodeBase64 = exports.$decodeBase64 = exports.$arrayBufferFromBytes = exports.$uint8ArrayFromDataLike = exports.$uint8ArrayFromBytes = exports.$bufferFromBytes = exports.$bytesFromBytes = exports.$bufferFromArrayBuffer = void 0;
const commons_1 = require("./commons");
const tscharset_1 = require("./tscharset");
const tsdata_1 = require("./tsdata");
function $bufferFromArrayBuffer(a) {
    return ArrayBuffer.isView(a) ? Buffer.from(a.buffer, a.byteOffset, a.byteLength) : Buffer.from(a);
}
exports.$bufferFromArrayBuffer = $bufferFromArrayBuffer;
function $bytesFromBytes(source, sourceStart, sourceEnd) {
    const [sourceLen, start, end,] = (0, commons_1.$lse)(source, sourceStart, sourceEnd);
    if ((source instanceof (Array))) {
        return start === 0 && end === sourceLen ? source : source.slice(start, end);
    }
    const ret = [];
    for (let i = start, j = 0; i < end; i++, j++) {
        ret[j] = source[i];
    }
    return ret;
}
exports.$bytesFromBytes = $bytesFromBytes;
function $bufferFromBytes(source, sourceStart, sourceEnd) {
    const [sourceLen, start, end, len] = (0, commons_1.$lse)(source, sourceStart, sourceEnd);
    if (source instanceof Buffer) {
        return start === 0 && end === sourceLen ? source : source.subarray(start, end);
    }
    else if (start === 0 && end === sourceLen) {
        return Buffer.from(source);
    }
    const ret = Buffer.allocUnsafe(len);
    for (let i = start, j = 0; i < end; i++, j++) {
        ret[j] = source[i];
    }
    return ret;
}
exports.$bufferFromBytes = $bufferFromBytes;
function $uint8ArrayFromBytes(source, sourceStart, sourceEnd) {
    const [sourceLen, start, end, len] = (0, commons_1.$lse)(source, sourceStart, sourceEnd);
    if (source instanceof Uint8Array && !(source instanceof Buffer)) {
        return start === 0 && end === sourceLen ? source : source.subarray(start, end);
    }
    const ret = new Uint8Array(len);
    for (let i = start, j = 0; i < end; i++, j++) {
        ret[j] = source[i];
    }
    return ret;
}
exports.$uint8ArrayFromBytes = $uint8ArrayFromBytes;
function $uint8ArrayFromDataLike(source, sourceStart, sourceEnd) {
    return source instanceof ArrayBuffer ?
        $uint8ArrayFromBytes($bufferFromArrayBuffer(source), sourceStart, sourceEnd) :
        (source instanceof tsdata_1.TSData ?
            source.toUint8Array(sourceStart, sourceEnd) :
            $uint8ArrayFromBytes(source, sourceStart, sourceEnd));
}
exports.$uint8ArrayFromDataLike = $uint8ArrayFromDataLike;
function $arrayBufferFromBytes(source, sourceStart, sourceEnd) {
    const [, start, end, len] = (0, commons_1.$lse)(source, sourceStart, sourceEnd);
    const ret = new ArrayBuffer(len);
    if (len > 0) {
        const view = new Uint8Array(ret);
        for (let i = start, j = 0; i < end; i++, j++) {
            view[j] = source[i];
        }
    }
    return ret;
}
exports.$arrayBufferFromBytes = $arrayBufferFromBytes;
const base64KeyStr = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
function $decodeBase64(input, reference = base64KeyStr) {
    var chr1, chr2, chr3;
    var enc1, enc2, enc3, enc4;
    var i = 0;
    var size = 0;
    const len = input.length;
    input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");
    var uint8 = new Uint8Array(input.length);
    while (i < len) {
        enc1 = reference.indexOf(input.charAt(i++));
        enc2 = reference.indexOf(input.charAt(i++));
        enc3 = reference.indexOf(input.charAt(i++));
        enc4 = reference.indexOf(input.charAt(i++));
        chr1 = (enc1 << 2) | (enc2 >> 4);
        chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
        chr3 = ((enc3 & 3) << 6) | enc4;
        uint8[size++] = (chr1 & 0xff);
        if (enc3 !== 64) {
            uint8[size++] = (chr2 & 0xff);
        }
        if (enc4 !== 64) {
            uint8[size++] = (chr3 & 0xff);
        }
    }
    return uint8.subarray(0, size);
}
exports.$decodeBase64 = $decodeBase64;
function $encodeBase64(source, reference = base64KeyStr) {
    var output = "";
    var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
    const input = (0, commons_1.$isstring)(source) ? tscharset_1.TSCharset.binaryCharset().uint8ArrayFromString(source) : source;
    const len = input.length;
    var i = 0;
    while (i < len) {
        chr1 = input[i++];
        chr2 = input[i++];
        chr3 = input[i++];
        enc1 = chr1 >> 2;
        enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
        enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
        enc4 = chr3 & 63;
        if (isNaN(chr2)) {
            enc3 = enc4 = 64;
        }
        else if (isNaN(chr3)) {
            enc4 = 64;
        }
        output = output + reference.charAt(enc1) + reference.charAt(enc2) + reference.charAt(enc3) + reference.charAt(enc4);
    }
    return output;
}
exports.$encodeBase64 = $encodeBase64;
function $dataAspect(source, opts = { prefix: '[', suffix: ']', separator: ',', showLength: true }) {
    var _a, _b, _c;
    const fn = (0, commons_1.$isfunction)(opts.transformFn) ? opts.transformFn : (n) => n.toString();
    const n = source.length;
    let s = (0, commons_1.$ok)(opts.name) ? opts.name : source.constructor.name;
    if (opts.showLength) {
        s += `(${n})`;
    }
    if (s.length > 0) {
        s += ' ';
    }
    if ((_a = opts.prefix) === null || _a === void 0 ? void 0 : _a.length) {
        s += opts.prefix;
    }
    ;
    if (n > 0) {
        if ((_b = opts.prefix) === null || _b === void 0 ? void 0 : _b.length) {
            s += ' ';
        }
        ;
        s += fn(source[0]);
        for (let i = 1; i < n; i++) {
            s += opts.separator + ' ' + fn(source[i]);
        }
    }
    return ((_c = opts.suffix) === null || _c === void 0 ? void 0 : _c.length) ? s + ` ${opts.suffix}` : s;
}
exports.$dataAspect = $dataAspect;
Buffer.prototype.leafInspect = function leafInspect() { return '<' + $dataAspect(this, { prefix: '', suffix: '', separator: '', showLength: false, transformFn: (n) => n.toHex2() }) + '>'; };
Uint8Array.prototype.leafInspect = function leafInspect() { return $dataAspect(this); };
Uint16Array.prototype.leafInspect = function leafInspect() { return $dataAspect(this); };
Uint32Array.prototype.leafInspect = function leafInspect() { return $dataAspect(this); };
ArrayBuffer.prototype.leafInspect = function leafInspect() {
    const buf = $bufferFromArrayBuffer(this);
    return 'ArrayBuffer { [Uint8Contents]: <' + $dataAspect(buf, { prefix: '', suffix: '', separator: '', showLength: false, name: '', transformFn: (n) => n.toHex2(true) }) + '>, byteLength: ' + buf.length + ' }';
};
String.prototype.toBase64 = function toBase64() { return $encodeBase64(this); };
Uint8Array.prototype.toBase64 = function toBase64() { return $encodeBase64(this); };
ArrayBuffer.prototype.toBase64 = function toBase64() { return $encodeBase64($bufferFromArrayBuffer(this)); };
//# sourceMappingURL=data.js.map