"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HTMLContent = exports.$HTML = exports.$capitalize = exports.$firstcap = exports.$normspaces = exports.$lines = exports.$right = exports.$left = exports.$trim = exports.$ftrim = exports.$rtrim = exports.$ltrim = exports.$ascii = void 0;
const commons_1 = require("./commons");
const string_tables_1 = require("./string_tables");
const tsdate_1 = require("./tsdate");
function $ascii(source) {
    const l = (0, commons_1.$length)(source);
    if (!l)
        return '';
    let s = source.replace(/≠/g, "");
    s = s.normalize("NFD").replace(/[\u0300-\u036f]|\u00a8|\u00b4/g, "").normalize("NFKD");
    return s.replace(/[^\x00-\x7F]/g, x => string_tables_1.FoundationASCIIConversion[x] || '');
}
exports.$ascii = $ascii;
function $ltrim(s) { return (0, commons_1.$length)(s) ? s.replace(string_tables_1.FoundationLeftTrimRegex, "") : ''; }
exports.$ltrim = $ltrim;
function $rtrim(s) { return (0, commons_1.$length)(s) ? s.replace(string_tables_1.FoundationRightTrimRegex, "") : ''; }
exports.$rtrim = $rtrim;
function $ftrim(s) { return (0, commons_1.$length)(s) ? s.replace(string_tables_1.FoundationLeftTrimRegex, "").replace(string_tables_1.FoundationRightTrimRegex, "") : ''; }
exports.$ftrim = $ftrim;
exports.$trim = $ftrim;
function $left(source, leftPart) {
    const l = (0, commons_1.$length)(source);
    let n = (0, commons_1.$unsigned)(leftPart);
    if (!n)
        (n = 1);
    return l > 0 ? (n >= l ? source : source.slice(0, n)) : '';
}
exports.$left = $left;
function $right(source, rightPart) {
    const l = (0, commons_1.$length)(source);
    let n = (0, commons_1.$unsigned)(rightPart);
    if (!n)
        (n = 1);
    return l > 0 ? (n >= l ? source : source.slice(l - n, l)) : '';
}
exports.$right = $right;
function $lines(s, useOnlyASCIISeparators = false) {
    const len = (0, commons_1.$length)(s);
    const ret = [];
    if (len > 0) {
        let lastWasCR = false, lastWasNewLine = false;
        let last = 0;
        let i = 0;
        const [CR, LF, FF, NEL, LS, PS] = [0x000D, 0x000A, 0x000C, 0x0085, 0x02028, 0x02029];
        const isOtherLineSeparator = useOnlyASCIISeparators ? (_) => false : (c) => c === LS || c === PS || c === NEL || c === FF;
        while (i < len) {
            const c = s.charCodeAt(i);
            if (lastWasCR) {
                lastWasCR = false;
                ret.push(s.slice(last, i - 1));
                if (c === LF) {
                    last = i + 1;
                    lastWasNewLine = true;
                }
                else if (isOtherLineSeparator(c)) {
                    ret.push('');
                    last = i + 1;
                    lastWasNewLine = true;
                }
                else {
                    last = i;
                    i--;
                    lastWasNewLine = false;
                }
            }
            else if (c === CR) {
                lastWasCR = true;
                lastWasNewLine = false;
            }
            else if (c === LF || isOtherLineSeparator(c)) {
                ret.push(s.slice(last, i));
                last = i + 1;
                lastWasNewLine = true;
            }
            else {
                lastWasNewLine = false;
            }
            i++;
        }
        if (lastWasCR) {
            ret.push(s.slice(last, i - 1));
            ret.push('');
        }
        else if (lastWasNewLine) {
            ret.push('');
        }
        else if (last < i) {
            ret.push(s.slice(last, i));
        }
    }
    else if ((0, commons_1.$ok)(s)) {
        ret.push('');
    }
    return ret;
}
exports.$lines = $lines;
function $normspaces(s) { return $ftrim(s).replace(string_tables_1.FoundationFindAllWhitespacesRegex, " "); }
exports.$normspaces = $normspaces;
function $firstcap(s) { return _capitalize(s, 1); }
exports.$firstcap = $firstcap;
function $capitalize(s) { return _capitalize(s); }
exports.$capitalize = $capitalize;
function $HTML(s, reference = string_tables_1.FoundationHTMLEncoding) {
    const len = (0, commons_1.$length)(s);
    let ret = "";
    for (let i = 0; i < len; i++) {
        const c = s.charCodeAt(i);
        if (c <= 0xff) {
            ret += reference[c];
        }
        else {
            ret += '&#' + c;
        }
    }
    return ret;
}
exports.$HTML = $HTML;
class HTMLContent extends String {
}
exports.HTMLContent = HTMLContent;
String.prototype.ascii = function ascii() { return $ascii(this); };
String.prototype.capitalize = function capitalize() { return $capitalize(this); };
String.prototype.firstCap = function firstCap() { return $firstcap(this); };
String.prototype.ftrim = function ftrim() { return $ftrim(this); };
String.prototype.isDate = function isDate() { return (0, commons_1.$ok)((0, commons_1.$isodate)(this)); };
String.prototype.isEmail = function isEmail() { return (0, commons_1.$ok)((0, commons_1.$email)(this)); };
String.prototype.isNewLine = function isNewLine() { return string_tables_1.FoundationNewLineStringCodeSet.has(this); };
String.prototype.isStrictWhiteSpace = function isStrictWhiteSpace() { return string_tables_1.FoundationStrictWhiteSpacesStringCodeSet.has(this); };
String.prototype.isUrl = function isUrl() { return (0, commons_1.$ok)((0, commons_1.$url)(this)); };
String.prototype.isUUID = function isUUID() { return (0, commons_1.$ok)((0, commons_1.$UUID)(this)); };
String.prototype.isWhiteSpace = function isWhiteSpace() { return string_tables_1.FoundationWhiteSpacesStringCodeSet.has(this); };
String.prototype.left = function left(leftPart) { return $left(this, leftPart); };
String.prototype.lines = function lines(useOnlyASCIISeparators) { return $lines(this, useOnlyASCIISeparators); };
String.prototype.ltrim = function ltrim() { return $ltrim(this); };
String.prototype.normalizeSpaces = function normalizeSpaces() { return $normspaces(this); };
String.prototype.right = function right(rightPart) { return $right(this, rightPart); };
String.prototype.rtrim = function rtrim() { return $rtrim(this); };
String.prototype.singular = function singular() { return this.toUnsigned() === 1; };
String.prototype.toDate = function toDate() { return (0, commons_1.$isdate)(this) ? new Date(this) : null; };
String.prototype.toHTML = function toHTML() { return $HTML(this); };
String.prototype.toHTMLContent = function toHTMLContent() { return new HTMLContent(this); };
String.prototype.toInt = function toInt(defaultValue) { return (0, commons_1.$toint)(this, defaultValue); };
String.prototype.toTSDate = function toTSDate() { return tsdate_1.TSDate.fromIsoString(this); };
String.prototype.toUnsigned = function toUnsigned(defaultValue) { return (0, commons_1.$tounsigned)(this, defaultValue); };
HTMLContent.prototype.toHTML = function toHTML() { return $HTML('' + this, string_tables_1.FoundationHTMLStructureEncoding); };
function _capitalize(s, max = 0) {
    const len = (0, commons_1.$length)(s);
    let ret = "";
    if (!max) {
        max = len;
    }
    ;
    let lastCharWasNotLetter = true;
    let n = 0;
    for (let i = 0; i < len; i++) {
        const c = s.charAt(i);
        const isLetter = _charAssimilableAsLetter(c);
        if (isLetter && lastCharWasNotLetter && n < max) {
            ret += c.toUpperCase();
            n++;
        }
        else {
            ret += c;
        }
        lastCharWasNotLetter = !isLetter;
    }
    return ret;
}
function _charAssimilableAsLetter(c) {
    c = $ascii(c);
    if (c.length) {
        const v = c.charCodeAt(0) & ~32;
        return v >= 65 && v <= 90;
    }
    return false;
}
//# sourceMappingURL=strings.js.map