"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSCountry = void 0;
const commons_1 = require("./commons");
const tsdefaults_1 = require("./tsdefaults");
const types_1 = require("./types");
const countries_json_1 = __importDefault(require("./countries.json"));
const compare_1 = require("./compare");
const strings_1 = require("./strings");
const customInspectSymbol = Symbol.for('nodejs.util.inspect.custom');
class TSCountry {
    constructor(info, locales) {
        this.alpha2Code = info.alpha2Code;
        this.alpha3Code = info.alpha3Code;
        this.names = info.names;
        this.domains = info.domains;
        this.EEC = info.EEC;
        this.spokenLanguages = info.spokenLanguages;
        this.dialCode = info.dialCode;
        this.currency = info.currency;
        this.locales = Object.assign({}, locales);
    }
    static loadCountries(localesMap, managedLanguages) {
        if (!(0, commons_1.$ok)(TSCountry.__countriesMap)) {
            TSCountry.__countriesMap = new Map();
            TSCountry.__countries = [];
            countries_json_1.default.forEach(info => {
                var _a;
                let loc = (0, commons_1.$ok)(info.localeLanguage) ? localesMap.get(info.localeLanguage) : TSCountry.__EULocale;
                if (!(0, commons_1.$ok)(loc)) {
                    loc = TSCountry.__EULocale;
                }
                const c = new TSCountry(info, loc);
                c.locales.startingWeekDay = info.startingWeekDay;
                TSCountry.__countries.push(c);
                TSCountry.__countriesMap.set(c.alpha2Code, c).set(c.alpha3Code, c);
                managedLanguages.forEach(l => TSCountry.__countriesMap.set((0, strings_1.$ascii)(c.names[l].toUpperCase()), c));
                (_a = info.aliases) === null || _a === void 0 ? void 0 : _a.forEach(a => TSCountry.__countriesMap.set((0, strings_1.$ascii)(a.toUpperCase()), c));
            });
        }
    }
    static country(c) {
        c = (0, strings_1.$ftrim)(c);
        if (c.length) {
            if (!(0, commons_1.$ok)(TSCountry.__countriesMap)) {
                tsdefaults_1.TSDefaults.defaults();
            }
            const ret = TSCountry.__countriesMap.get((0, strings_1.$ascii)(c.toUpperCase()));
            if ((0, commons_1.$ok)(ret)) {
                return ret;
            }
        }
        return null;
    }
    static countries() {
        if (!(0, commons_1.$ok)(TSCountry.__countriesMap)) {
            tsdefaults_1.TSDefaults.defaults();
        }
        return [...TSCountry.__countries];
    }
    get name() { return this.names[(0, tsdefaults_1.$language)()]; }
    get label() { return this.names[types_1.Languages.en]; }
    get language() { return this.locales.language; }
    get languageName() { return this.locales.names[(0, tsdefaults_1.$language)()]; }
    clone() { return this; }
    translatedName(lang) {
        const ret = this.names[(0, commons_1.$ok)(lang) ? lang : (0, tsdefaults_1.$language)()];
        return (0, commons_1.$length)(ret) ? ret : null;
    }
    toString() { return this.alpha2Code; }
    toJSON() { return this.alpha2Code; }
    toArray() { return [this.alpha2Code, this.alpha3Code]; }
    isEqual(other) { return this === other; }
    compare(other) {
        if (this === other) {
            return types_1.Same;
        }
        if (other instanceof TSCountry) {
            return (0, compare_1.$compare)(this.alpha2Code, other.alpha2Code);
        }
        return undefined;
    }
    leafInspect() { return `<${this.label.capitalize()} (${this.alpha2Code})>`; }
    [customInspectSymbol](depth, inspectOptions, inspect) {
        return this.leafInspect();
    }
}
exports.TSCountry = TSCountry;
TSCountry.__EULocale = {
    language: types_1.Languages.en,
    names: { fr: "anglais", en: "english", de: "englisch", it: "inglese", es: "inglés", pt: "inglês", nl: "Engels" },
    months: ["January", "February", "March", "April", "May", "June", "July", "August", "Septembre", "Octobre", "Novembre", "Decembre"],
    shortMonths: ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"],
    days: ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
    shortDays: ["Sun.", "Mon.", "Tue.", "Wed.", "Thu.", "Fri.", "Sat."],
    startingWeekDay: 1,
    dateTimeFormat: "%d/%m/%Y %H:%M:%S",
    dateFormat: "%d/%m/%Y",
    shortDateFormat: "%d/%m/%y",
    shortDateTimeFormat: "%d/%m/%y %H:%M:%S",
    timeFormat: "%H:%M:%S",
    partialTimeFormat: "%H:%M",
    ampm: ['AM', 'PM']
};
//# sourceMappingURL=tscountry.js.map