"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSHttpError = exports.$subclassReponsabililty = exports.TSError = exports.TSUniqueError = void 0;
const commons_1 = require("./commons");
const utils_1 = require("./utils");
class TSUniqueError extends Error {
    constructor(message) {
        super(message);
        this.name = message;
    }
    static genericError() {
        if (!(0, commons_1.$ok)(this.__genericInstance)) {
            this.__genericInstance = new TSUniqueError('GenericSingletonError');
        }
        return this.__genericInstance;
    }
    static timeoutError() {
        if (!(0, commons_1.$ok)(this.__timeoutInstance)) {
            this.__timeoutInstance = new TSUniqueError('TimeoutSingletonError');
        }
        return this.__timeoutInstance;
    }
    leafInspect() { return this.name; }
}
exports.TSUniqueError = TSUniqueError;
class TSError extends Error {
    constructor(message, info) {
        super(message);
        this.info = info;
    }
    static assertIntParam(v, fn, param) {
        if ((0, commons_1.$ok)(v) && !(0, commons_1.$isint)(v)) {
            throw new TSError(`parameter '${param}' of ${fn}() must be an integer`, {
                fn: fn,
                param: param,
                value: v
            });
        }
    }
    static assertNotInBrowser(fn) {
        if ((0, utils_1.$inbrowser)()) {
            throw new TSError(`unavailable ${fn}() ${fn.includes('.') ? 'method' : 'function'} in browser`, { functionName: fn });
        }
    }
    static assertUnsignedParam(v, fn, param) {
        if ((0, commons_1.$ok)(v) && !(0, commons_1.$isunsigned)(v)) {
            throw new TSError(`parameter '${param}' of ${fn}() must be an unsigned`, {
                fn: fn,
                param: param,
                value: v
            });
        }
    }
    entries() { return Object.entries({ name: this.name, message: this.message, info: this.info }); }
}
exports.TSError = TSError;
function $subclassReponsabililty(instance, method) {
    const c = instance.constructor;
    if (c === Function) {
        throw new TSError(`implementation of static method ${instance.name}.${method.name}() is subclasses reponsabillity.`, {
            isStatic: true,
            object: instance,
            method: method
        });
    }
    throw new TSError(`implementation of method ${c.name}.${method.name}() is subclasses reponsabillity.`, {
        isStatic: false,
        object: instance,
        method: method
    });
}
exports.$subclassReponsabililty = $subclassReponsabililty;
class TSHttpError extends TSError {
    constructor(message, status, info) {
        super(message, info);
        this.status = status;
    }
    entries() { return Object.entries({ name: this.name, status: this.status, message: this.message, info: this.info }); }
}
exports.TSHttpError = TSHttpError;
//# sourceMappingURL=tserrors.js.map