/// <reference types="node" />
/// <reference types="node" />
import * as http from "http";
import { Verb } from "./tsrequest";
import { Nullable, StringDictionary, TSDictionary, uint16 } from "./types";
import { TSStaticWebSiteOptions } from "./tsservercomp";
import { TSDate } from "./tsdate";
import { TSColor } from "./tscolor";
export interface TSServerRequest {
    method: Verb;
    url: URL;
    parameters: TSParameterDictionary;
    query: TSQueryDictionary;
    message: http.IncomingMessage;
}
export type TSServerResponse = http.ServerResponse;
export type TSEndPointManager = (req: TSServerRequest, resp: TSServerResponse) => Promise<void>;
export interface TSEndPoint {
    manager: TSEndPointManager;
    query?: TSQueryDefinition;
}
export type TSEndPoints = {
    [key in Verb]?: TSEndPoint | TSEndPointManager;
};
export type TSEndPointsDictionary = TSDictionary<TSEndPoints | TSEndPoint | TSEndPointManager>;
export type TSEndPointParameter = string | number | boolean | TSDate | TSColor;
export type TSParameterDictionary = TSDictionary<TSEndPointParameter>;
export type TSQueryValue = TSEndPointParameter;
export type TSQueryDictionary = TSDictionary<TSQueryValue>;
export declare const TSParametricTokenType: {
    readonly string: "string";
    readonly number: "number";
    readonly int: "int";
    readonly unsigned: "unsigned";
    readonly boolean: "boolean";
    readonly date: "date";
    readonly color: "color";
    readonly uuid: "uuid";
    readonly email: "email";
};
export type TSParametricTokenType = typeof TSParametricTokenType[keyof typeof TSParametricTokenType];
export declare enum TSServerStartStatus {
    AlreadyRunning = 0,
    HTTP = 1,
    HTTPS = 2
}
export interface TSParametricToken {
    name: string;
    type: TSParametricTokenType;
}
export interface TSQueryItem {
    type: TSParametricTokenType;
    mandatory?: boolean;
}
export type TSQueryDefinition = TSDictionary<TSParametricTokenType | TSQueryItem>;
export interface TSServerOptions extends TSStaticWebSiteOptions {
    host?: string;
    port?: uint16;
    rootPath?: string;
    webSites?: StringDictionary;
    logInfo?: boolean;
    key?: Nullable<Buffer>;
    certificate?: Nullable<Buffer>;
    maxHeaderSize?: Nullable<number>;
    tlsSecTimeout?: Nullable<number>;
    handshakeMsTimeout?: Nullable<number>;
    forceCloseMsTimeout?: Nullable<number>;
}
export declare class TSServer {
    private static __server;
    private static readonly DEFAULT_CLOSE_TIMEOUT;
    readonly host: string;
    readonly port: number;
    readonly rootPath: string;
    readonly isHTTPs: boolean;
    private _serverOptions;
    private _httpServer;
    private _endPoints;
    private _sites;
    private _logger;
    private _logInfo;
    private _connections;
    private _terminating;
    private _forceCloseTimeout;
    static start(endPoints: Nullable<TSEndPointsDictionary>, opts?: Nullable<TSServerOptions>): Promise<TSServerStartStatus | Error>;
    static clearCaches(): Promise<void>;
    static stop(): Promise<Error | undefined>;
    private constructor();
    private _start;
    private _onConnection;
    private _onRequest;
    private _requestDidEnd;
    private _clearCaches;
    private _stop;
    private _internalStopServer;
}
export declare enum TSServerLogType {
    Log = "Info",
    Warning = "Warning",
    Error = "Error"
}
export type TSServerLogger = (server: TSServer, req: http.IncomingMessage | undefined, type: TSServerLogType, messages: string) => Promise<void>;
