"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TSServerLogType = exports.TSServer = exports.TSServerStartStatus = exports.TSParametricTokenType = void 0;
const http = __importStar(require("http"));
const https = __importStar(require("https"));
const commons_1 = require("./commons");
const tserrors_1 = require("./tserrors");
const tsrequest_1 = require("./tsrequest");
const types_1 = require("./types");
const utils_1 = require("./utils");
const tsservercomp_1 = require("./tsservercomp");
const strings_1 = require("./strings");
const fs_1 = require("./fs");
exports.TSParametricTokenType = {
    string: 'string',
    number: 'number',
    int: 'int',
    unsigned: 'unsigned',
    boolean: 'boolean',
    date: 'date',
    color: 'color',
    uuid: 'uuid',
    email: 'email'
};
var TSServerStartStatus;
(function (TSServerStartStatus) {
    TSServerStartStatus[TSServerStartStatus["AlreadyRunning"] = 0] = "AlreadyRunning";
    TSServerStartStatus[TSServerStartStatus["HTTP"] = 1] = "HTTP";
    TSServerStartStatus[TSServerStartStatus["HTTPS"] = 2] = "HTTPS";
})(TSServerStartStatus = exports.TSServerStartStatus || (exports.TSServerStartStatus = {}));
class TSServer {
    static start(endPoints, opts) {
        return __awaiter(this, void 0, void 0, function* () {
            if (TSServer.__server) {
                return TSServerStartStatus.AlreadyRunning;
            }
            if ((0, commons_1.$objectcount)(endPoints) === 0 && (0, commons_1.$objectcount)(opts === null || opts === void 0 ? void 0 : opts.webSites) === 0) {
                return new tserrors_1.TSError('TSServer must at least serve one static website or one end point', {
                    endPoints: endPoints,
                    options: opts
                });
            }
            try {
                TSServer.__server = new TSServer((0, commons_1.$value)(endPoints, {}), (0, commons_1.$value)(opts, {}));
            }
            catch (e) {
                return e;
            }
            yield TSServer.__server._start();
            return TSServer.__server.isHTTPs ? TSServerStartStatus.HTTPS : TSServerStartStatus.HTTP;
        });
    }
    static clearCaches() {
        return __awaiter(this, void 0, void 0, function* () { if (TSServer.__server) {
            yield TSServer.__server._clearCaches();
        } });
    }
    static stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (TSServer.__server) {
                const e = yield TSServer.__server._stop();
                if (!(0, commons_1.$ok)(e)) {
                    delete TSServer.__server;
                    return undefined;
                }
                return e;
            }
            return undefined;
        });
    }
    constructor(endPoints, opts = {}) {
        this.rootPath = '';
        this.isHTTPs = false;
        this._serverOptions = {};
        this._connections = new Map;
        this._terminating = false;
        this._logger = (0, commons_1.$ok)(opts.logger) ? opts.logger : _internalLogger;
        this._logInfo = !!opts.logInfo;
        this.isHTTPs = (0, commons_1.$length)(opts.key) > 0 && (0, commons_1.$length)(opts.certificate) > 0;
        if (this.isHTTPs) {
            this._serverOptions.cert = opts.certificate;
            this._serverOptions.key = opts.key;
            let t = (0, commons_1.$unsigned)(opts.tlsSecTimeout);
            if (t > 0) {
                this._serverOptions.sessionTimeout = t;
            }
            t = (0, commons_1.$unsigned)(opts.handshakeMsTimeout);
            if (t > 0) {
                this._serverOptions.handshakeTimeout = t;
            }
        }
        const n = (0, commons_1.$unsigned)(opts.maxHeaderSize);
        if (n > 0) {
            this._serverOptions.maxHeaderSize = n;
        }
        const ft = (0, commons_1.$unsigned)(opts.forceCloseMsTimeout);
        this._forceCloseTimeout = ft === 0 ? TSServer.DEFAULT_CLOSE_TIMEOUT : ft;
        const rp = (0, strings_1.$ftrim)(opts.rootPath);
        if (rp.length > 1 && (0, fs_1.$isabsolutepath)(rp)) {
            this.rootPath = rp;
        }
        this._sites = [];
        if ((0, commons_1.$ok)(opts.webSites)) {
            const keys = (0, commons_1.$keys)(opts.webSites);
            if ((0, utils_1.$inbrowser)() && keys.length) {
                throw new tserrors_1.TSError('TSServer cannot handle static websides inside a browser', { endPoints: endPoints, options: opts });
            }
            keys.forEach(u => {
                this._sites.push(new tsservercomp_1.TSStaticWebsite(u, opts.webSites[u], {
                    logger: this._logger,
                    managedTypes: opts.managedTypes,
                    maxCacheSize: opts.maxCacheSize,
                    maxCachedFileSize: opts.maxCachedFileSize,
                    maxCachedFiles: opts.maxCachedFiles,
                    maxBlacklistedFiles: opts.maxBlacklistedFiles
                }));
            });
        }
        this._endPoints = [];
        (0, commons_1.$keys)(endPoints).forEach(path => {
            this._endPoints.push(new tsservercomp_1.TSParametricEndPoints(path, endPoints[path], true));
        });
        if ((0, commons_1.$ok)(opts.port)) {
            if (!(0, commons_1.$isunsigned)(opts.port, types_1.UINT16_MAX)) {
                throw new tserrors_1.TSError(`TSServer.constructor(): Bad ${this.isHTTPs ? 'HTTPS' : 'HTTP'} server port ${opts.port}.`, {
                    endPoints: endPoints,
                    options: opts
                });
            }
            this.port = opts.port;
        }
        else {
            this.port = 3000;
        }
        this.host = (0, strings_1.$ftrim)(opts.host);
        if (!this.host.length) {
            this.host = `${this.isHTTPs ? 'https' : 'http'}://localhost/`;
        }
    }
    _start() {
        return __awaiter(this, void 0, void 0, function* () {
            const managementCallBack = (req, res) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const method = tsservercomp_1.TSParametricEndPoints.validRequestMethod(req.method);
                    const url = new URL((0, commons_1.$string)(req.url), this.host);
                    if (!(0, commons_1.$ok)(method)) {
                        throw new tserrors_1.TSHttpError(`Request method '${req.method}' is not allowed.`, tsrequest_1.Resp.NotAllowed, {
                            method: req.method,
                            path: (0, commons_1.$length)(url.pathname) ? url.pathname : '/'
                        });
                    }
                    if (!(0, commons_1.$length)(url.pathname) || url.pathname === '/') {
                        if (this.rootPath.length) {
                            url.pathname = this.rootPath;
                        }
                        else {
                            throw new tserrors_1.TSHttpError('Root path is not Accessible', tsrequest_1.Resp.Forbidden, {
                                method: req.method,
                                path: '/'
                            });
                        }
                    }
                    let pep = undefined;
                    let parameters = {};
                    this._endPoints.forEach(ep => {
                        const params = ep.parametersFromPath(url.pathname);
                        if ((0, commons_1.$ok)(params) && (!(0, commons_1.$defined)(pep) || pep.depth < ep.depth)) {
                            pep = ep;
                            parameters = params;
                        }
                    });
                    if ((0, commons_1.$ok)(pep)) {
                        yield pep.execute({
                            url: url,
                            method: method,
                            parameters: parameters,
                            message: req,
                            query: {}
                        }, res);
                        if (this._logInfo) {
                            yield this._logger(this, req, TSServerLogType.Log, `did handle resource '${url.pathname}'.`);
                        }
                        return;
                    }
                    else if (method === tsrequest_1.Verb.Get) {
                        for (let s of this._sites) {
                            const [b, type] = s.getStaticResource(url.pathname);
                            if ((0, commons_1.$ok)(b)) {
                                if (this._logInfo) {
                                    yield this._logger(this, req, TSServerLogType.Log, `did handle static resource '${url.pathname}'.`);
                                }
                                res.setHeader('Content-Type', type);
                                res.writeHead(tsrequest_1.Resp.OK);
                                res.end(b);
                                return;
                            }
                        }
                    }
                    throw new tserrors_1.TSHttpError(`endpoint ${method} '${url.pathname}' was not found.`, tsrequest_1.Resp.NotFound, {
                        method: method,
                        path: (0, commons_1.$length)(url.pathname) ? url.pathname : '/'
                    });
                }
                catch (e) {
                    let ret = {};
                    ret.status = (0, commons_1.$isunsigned)(e === null || e === void 0 ? void 0 : e.status) && Object.values(tsrequest_1.Resp).includes(e.status) ? e.status : tsrequest_1.Resp.InternalError;
                    ret.error = e.message;
                    if (!(0, commons_1.$length)(ret.error)) {
                        ret.error = 'Unknown internal Error';
                    }
                    ;
                    if ((0, commons_1.$ok)(e.info)) {
                        ret.info = e.info;
                    }
                    yield this._logger(this, req, TSServerLogType.Warning, `${ret.status} - ${ret.error}`);
                    res.writeHead(ret.status, { 'Content-Type': 'application/json' });
                    res.end(JSON.stringify(ret));
                }
            });
            this._httpServer = this.isHTTPs ?
                https.createServer(this._serverOptions, managementCallBack) :
                http.createServer(this._serverOptions, managementCallBack);
            this._httpServer.on('connection', this._onConnection.bind(this));
            this._httpServer.on('request', this._onRequest.bind(this));
            this._httpServer.listen(this.port, () => __awaiter(this, void 0, void 0, function* () {
                if (this._logInfo) {
                    yield this._logger(this, undefined, TSServerLogType.Log, `running on port ${this.port} '${this.host}' ...`);
                }
            }));
        });
    }
    _onConnection(socket) {
        this._connections.set(socket, { refcount: 0, closeHeaderSent: false });
        socket.once('close', () => this._connections.delete(socket));
    }
    _onRequest(request, response) {
        const socket = request.socket;
        const info = this._connections.get(socket);
        if (info) {
            info.refcount++;
            if (this._terminating && !response.headersSent) {
                response.setHeader('Connection', 'close');
                info.closeHeaderSent = true;
            }
        }
        response.on('finish', () => { this._requestDidEnd(request); });
    }
    _requestDidEnd(request) {
        const socket = request.socket;
        const info = this._connections.get(socket);
        if (info) {
            info.refcount--;
            if (this._terminating && info.refcount === 0 && info.closeHeaderSent) {
                socket.end();
            }
        }
    }
    _clearCaches() {
        return __awaiter(this, void 0, void 0, function* () { for (let s of this._sites) {
            yield s.clearCaches();
        } });
    }
    _stop() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._logInfo) {
                yield this._logger(this, undefined, TSServerLogType.Log, `server is exiting...`);
            }
            if ((0, commons_1.$ok)(this._httpServer)) {
                const ret = yield this._internalStopServer();
                if ((0, commons_1.$ok)(ret)) {
                    yield this._logger(this, undefined, TSServerLogType.Error, `cannot stop for reason ${ret.name}:\n${ret.message}`);
                }
                return ret;
            }
            ;
            return undefined;
        });
    }
    _internalStopServer() {
        return __awaiter(this, void 0, void 0, function* () {
            if ((0, commons_1.$ok)(this._httpServer)) {
                this._terminating = true;
                const ret = new Promise((resolve, reject) => {
                    this._httpServer.close((error) => {
                        if ((0, commons_1.$ok)(error)) {
                            return reject(error);
                        }
                        resolve(undefined);
                    });
                });
                this._connections.forEach((info, socket) => {
                    if (info.refcount === 0) {
                        socket.end();
                    }
                });
                setTimeout(() => {
                    this._connections.forEach((_, socket) => {
                        if ((0, commons_1.$ismethod)(socket, 'destroy')) {
                            socket.destroy();
                        }
                        else {
                            socket.end();
                        }
                    });
                    this._connections.clear();
                }, this._forceCloseTimeout);
                return ret;
            }
            return undefined;
        });
    }
}
exports.TSServer = TSServer;
TSServer.__server = undefined;
TSServer.DEFAULT_CLOSE_TIMEOUT = 15000;
var TSServerLogType;
(function (TSServerLogType) {
    TSServerLogType["Log"] = "Info";
    TSServerLogType["Warning"] = "Warning";
    TSServerLogType["Error"] = "Error";
})(TSServerLogType = exports.TSServerLogType || (exports.TSServerLogType = {}));
;
const _internalLogger = (server, _, type, message) => __awaiter(void 0, void 0, void 0, function* () {
    (0, utils_1.$logterm)(`&0&xfoundation-ts[&w${server.isHTTPs ? 'https' : 'http'} server&x:${server.port}]-` + __TSServerlogHeaders[type] + "&0 &w" + message + "&0");
});
const __TSServerlogHeaders = {
    'Info': "&C&w LOGGING ",
    'Warning': "&O&w WARNING ",
    'Error': "&R&w  ERROR  "
};
//# sourceMappingURL=tsserver.js.map